package com.me.brickbuster.entity.powerup;

import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.glutils.ShapeRenderer;
import com.badlogic.gdx.math.Vector2;
import com.me.brickbuster.BrickBuster;
import com.me.brickbuster.Utils;
import com.me.brickbuster.entity.Brick;
import com.me.brickbuster.entity.Entity;
import net.dermetfan.utils.Pair;

public abstract class PowerUp extends Entity {

	public static final int RADIUS = 10;
	public static final int FALL_SPEED = 100;

	private Color color;
	private boolean isCaught;

	public PowerUp(BrickBuster brickBuster, Brick brick, Color color) {
		super(brickBuster, brick.getX() + Brick.BLOCK_WIDTH/2, brick.getY() + Brick.BLOCK_HEIGHT/2);
		this.color = color;
	}

	@Override
	public void render() {
		getShapeRenderer().begin(ShapeRenderer.ShapeType.Filled);
		getShapeRenderer().setColor(color);
		getShapeRenderer().circle(getX(), getY(), RADIUS);
		getShapeRenderer().end();
	}

	@Override
	public void update(float dt) {
		setY(getY() - FALL_SPEED * dt);

		Pair<Vector2, Vector2> paddle = getBrickBuster().getPaddle().getTopEdge();
		Vector2 lineDir = paddle.getValue().sub(paddle.getKey());
		Vector2 nearest = Utils.nearestPoint(paddle.getKey().cpy(), lineDir, getPos().cpy());

		if (nearest.dst(getX(), getY()) <= RADIUS) {
			activate();
			isCaught = true;
		}

		if (getY() + RADIUS < 0) {
			isCaught = true;
		}
	}

	public abstract void activate();

	public boolean isCaught() {
		return isCaught;
	}
}
